#<profile4xi profile = "03" />
#<program4xi program = "resourcesharer4xi.py">
#<abstract4xi abstract = "providing class ResourceSharer4Xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ResourceSharer4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import signal
    import socket
    import struct
    import sys
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # rare ones
    pass       
except:
    pass 
try:                 # imported-as ones 
    from pickle import _Pickler as Pickler         
except:
    pass 
               
   
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import reduction
try:                             # Python ones
    from multiprocessing.reduction import ResourceSharer
except:
    print("resourcesharer4xi.py *** failed from multiprocessing.reduction import ResourceSharer ***") 
try:
    from multiprocessing.util import is_exiting, sub_warning
except:
    print("resourcesharer4xi.py *** failed from multiprocessing.util import is_exiting, sub_warning ***")    
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from client4xi import Client4Xi
from current_process4xi import get_authkey_current_process4xi
from examine4xi import examine4xi
from is_exiting4xi import is_exiting4xi
from listener4xi import Listener4Xi
from register_after_fork4xi import register_after_fork4xi
from sub_warning4xi import sub_warning4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
#"
#/>

# =================================================================================================
#<class4xi class = "ResourceSharer4Xi" base = "object" child = "None" synopsis = "Server sharing registered resources with clients" usage = "
#   reduction.py:
#       class DupSocket(object):
#           __init__(self, sock)
#               self._id = resource_sharer.register(send, new_sock.close)
#   resource_sharer = ResourceSharer()    
#
#   duplicate4xi.py
#       from resourcesharer4xi import ResourceSharer4Xi
#       class DupHandle4Xi
#           resource_sharer2xi = ResourceSharer4Xi() # shared class variable
#           __init__
#               self._id = resource_sharer2xi.register(send, new_sock.close)
#>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class ResourceSharer4Xi(object): 
    '''
    Server sharing registered resources with clients
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('register', 'get_connection', 'stop', '_afterfork', '_start', '_serve' 
                 'sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ResourceSharer4Xi" usage = "initializes ResourceSharer4Xi with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self):
        '''
        initializes ResourceSharer4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/10a) started" + "; init-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 15:29:48 03/14/13 FLE Standard Time 
        
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 

            ResourceSharer.__init__(self)
            
            message2print = "{(StepOriginal) performed" + "; Original-ResourceSharer-performed-by-init-of-Listener4Xi: " + str(ResourceSharer.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #        
        else:        
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/4) - PASSES            
            # -----------------------------------------------------------------------------------------    
            self._key = 0
            message2print = "{(Step 2/4) set" + "; key-set-by-init-of-ResourceSharer4Xi to: " + str(self._key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0;
            
            
            self._cache = {}
            message2print = "{(Step 2/4) set" + "; cache-set-by-init-of-ResourceSharer4Xi to: " + str(self._cache)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {}            
            
            self._old_locks = []
            message2print = "{(Step 2/4) set" + "; old_locks-set-by-init-of-ResourceSharer4Xi to: " + str(self._old_locks)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # []             
            
            self._lock = threading.Lock() 
            message2print = "{(Step 2/4) set" + "; lock-set-by-init-of-ResourceSharer4Xi to: " + str(self._lock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <_thread.lock object at 0x00000000035A1A30>            
            
            self._listener = None
            message2print = "{(Step 2/4) set" + "; listener-set-by-init-of-ResourceSharer4Xi to: " + str(self._listener)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # None            
            
            self._address = None
            message2print = "{(Step 2/4) set" + "; address-set-by-init-of-ResourceSharer4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # None            
            
            self._thread = None
            message2print = "{(Step 2/4) set" + "; thread-set-by-init-of-ResourceSharer4Xi to: " + str(self._thread)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # None            
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/4) - PASSES            
            # -----------------------------------------------------------------------------------------    
            register_after_fork4xi(self, ResourceSharer4Xi._afterfork)
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4) - PASSES 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/4a) activated" + "; resourcesharer2xi-of-ResourceSharer4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 10:57:10 03/13/13 Pacific Daylight Time                
            
            message2print = "{(Step 4/4b) inherited" + "; object-inherited-by-resourcesharer2xi-of-ResourceSharer4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>   
                     
            message2print = "{(Step 4/4c) returning" + "; init-of-ResourceSharer4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 15:29:48 03/14/13 FLE Standard Time                       
                
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "register-of-ResourceSharer4Xi" usage = "registers with (self, send, close)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def register(self, send, close):
        '''
        registers (self, send, close)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/2) - NOT PERFORMED            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "register"
        
        message2print = "{(Step 1/2a) started" + "; register-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
        message2print = "{(Step 1/2b) passed" + "; send-passed-to-register-of-ResourceSharer4Xi: " + str(send) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
        
        message2print = "{(Step 1/2c) passed" + "; close-passed-to-register-of-ResourceSharer4Xi: " + str(close) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #            

        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-register-of-Listener4Xi: " + str(ResourceSharer.register) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #     
            
            return ResourceSharer.register(self, send, close)
            
           
        else:
                
            # -----------------------------------------------------------------------------------------
            # (Step 2/2)            
            # -----------------------------------------------------------------------------------------                                     
            with self._lock:
                if self._address is None:
                    message2print = "{(Step 2/2a) starting" + "; resourcesharer2xi-starting-by-register-of-ResourceSharer4Xi: " + str(self._start) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #     
                    self._start()
                    
                    message2print = "{(Step 2/2b) started" + "; resourcesharer2xi-started-by-register-of-ResourceSharer4Xi: " + str(self._start) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                else:
                    message2print = "{(Step 2/2c) using" + "; address-using-by-register-of-ResourceSharer4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #       
                    
                self._key += 1
                
                message2print = "{(Step 2/2c) added" + "; key-added-by-register-of-ResourceSharer4Xi: " + str(self._key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #   
                
                self._cache[self._key] = (send, close)
                
                message2print = "{(Step 2/2d) set" + "; cache-set-by-register-of-ResourceSharer4Xi to : " + str(self._cache[self._key]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #  
                
                message2print = "{(Step 2/2e) returning" + "; address-returning-by-register-of-ResourceSharer4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
                
                message2print = "{(Step 2/2f) returning" + "; key-returning-by-register-of-ResourceSharer4Xi: " + str(self._key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
                
                
                message2print = "{(Step 2/2g) returning" + "; register-of-ResourceSharer4Xi: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                #     
                
                return (self._address, self._key)
      #</methodize4xi> 
       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get_connection-of-ResourceSharer4Xi" usage = "get_connections with (ident)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    @staticmethod
    def get_connection(ident):
        '''
        get_connections with (ident)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4) - NOT PERFORMED            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "get_connection"
        
        message2print = "{(Step 1/4a) started" + "; get_connection-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
        message2print = "{(Step 1/4b) passed" + "; ident-passed-to-get_connection-of-ResourceSharer4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-get-connection-of-Listener4Xi: " + str(ResourceSharer.get_connection) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #     
            
            return ResourceSharer.get_connection(ident)
            
           
        else:
                        
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/4)            
            # -----------------------------------------------------------------------------------------                                     
            address2xi, key2xi = ident
            
            message2print = "{(Step 2/4a) set" + "; address2xi-set-by-get_connection-of-ResourceSharer4Xi to: " + str(address2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 2/4b) set" + "; key2xi-set-by-get_connection-of-ResourceSharer4Xi to: " + str(key2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/4)            
            # -----------------------------------------------------------------------------------------                                     
            authkey2xi = get_authkey_current_process4xi() 
            
            message2print = "{(Step 3/4a) set" + "; authkey2xi-set-by-get_connection-of-ResourceSharer4Xi to: " + str(authkey2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 3/4b) passing" + "; address2xi-passing-to-Client4Xi-by-get_connection-of-ResourceSharer4Xi: " + str(address2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
            
            message2print = "{(Step 3/4c) passing" + "; authkey2xi-passing-to-Client4Xi-by-get_connection-of-ResourceSharer4Xi: " + str(authkey2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                        
            c2xi = Client4Xi(address2xi, authkey=authkey2xi)
            message2print = "{(Step 3/4d) got" + "; c2xi-got-by-get_connection-of-ResourceSharer4Xi: " + str(c2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            pid2xi = os.getpid()
            message2print = "{(Step 3/4e) got" + "; pid2xi-got-by-get_connection-of-ResourceSharer4Xi: " + str(pid2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 3/4f) passing" + "; key2xi-passing-to-send-of-c2xi-by-get_connection-of-ResourceSharer4Xi: " + str(key2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
            
            message2print = "{(Step 3/4g) passing" + "; pid2xi-passing-to-send-of-c2xi-by-get_connection-of-ResourceSharer4Xi: " + str(pid2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #            
            
            c2xi.send((key2xi, pid2xi))
            
            message2print = "{(Step 3/4h) performed" + "; send-of-c2xi-performed-by-get_connection-of-ResourceSharer4Xi: " + str(c2xi.send) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/4)            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 4/4h) returning" + "; c2xi-returning-by-get_connection-of-ResourceSharer4Xi: " + str(c2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            return c2xi
        
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "stop-of-ResourceSharer4Xi" usage = "stops with (self, timeout=None)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def stop(self, timeout=None):
        '''
        stops with (self, timeout=None)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3) - NOT PERFORMED           
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "stop"
        
        message2print = "{(Step 1/3a) started" + "; stop-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
        message2print = "{(Step 1/3b) passed" + "; timeout-passed-to-stop-of-ResourceSharer4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-stop-of-Listener4Xi: " + str(ResourceSharer.stop) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
            ResourceSharer.stop(self, timeout)
            
            message2print = "{(StepOriginal) performed" + "; Original-ResourceSharer-performed-by-stop-of-Listener4Xi: " + str(ResourceSharer.stop) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #                
        else:
                             
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)            
            # -----------------------------------------------------------------------------------------                                     
            message2print = "{(Step 2/3a) using" + "; lock-using-by-stop-of-ResourceSharer4Xi: " + str(self._lock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
            
            with self._lock:
                if self._address is not None:
                    message2print = "{(Step 2/3b) using" + "; address-using-by-stop-of-ResourceSharer4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #
                
                    authkey2xi = get_authkey_current_process4xi() 
            
                    message2print = "{(Step 2/3b) got" + "; authkey2xi-got-by-stop-of-ResourceSharer4Xi to: " + str(authkey2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    message2print = "{(Step 2/3c) passing" + "; address-passing-to-Client4Xi-by-stop-of-ResourceSharer4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #
            
                    message2print = "{(Step 2/3d) passing" + "; authkey2xi-passing-to-Client4Xi-by-stop-of-ResourceSharer4Xi: " + str(authkey2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #
                    
                    c2xi = Client4Xi(self._address, authkey=authkey2xi)
                    
                    message2print = "{(Step 2/3e) got" + "; c2xi-got-by-stop-of-ResourceSharer4Xi to: " + str(c2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    c2xi.send(None)
                    message2print = "{(Step 2/3e) performed" + "; send-of-c2xi-performed-by-stop-of-ResourceSharer4Xi to: " + str(c2xi.send)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    c2xi.close()
                    message2print = "{(Step 2/3f) performed" + "; close-of-c2xi-performed-by-stop-of-ResourceSharer4Xi to: " + str(c2xi.close)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    message2print = "{(Step 2/3g) passing" + "; timeout-passing-to-join-of-thread-by-stop-of-ResourceSharer4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #
                    
                    self._thread.join(timeout)
                    
                    message2print = "{(Step 2/3h) performed" + "; join-of-thread-performed-by-stop-of-ResourceSharer4Xi to: " + str(self._thread.join)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    
                    if self._thread.is_alive():
                        warning2xi = 'stop-of-ResourceSharer4Xi: thread did not stop when asked'
                        sub_warning4xi(warning2xi)
                        message2print = "{(Step 2/3i) warned" + "; thread-not-stopped-warned-by-stop-of-ResourceSharer4Xi to: " + str(warning2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #
                        
                    self._listener.close()
                    
                    message2print = "{(Step 2/3j) performed" + "; close-of-listener-performed-by-stop-of-ResourceSharer4Xi to: " + str(self._listener.close)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    self._thread = None
                    message2print = "{(Step 2/3k) set" + "; thread-set-by-stop-of-ResourceSharer4Xi to: " + str(self._thread)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    self._address = None
                    message2print = "{(Step 2/3l) set" + "; address-set-by-stop-of-ResourceSharer4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    self._listener = None
                    message2print = "{(Step 2/3m) set" + "; listener-set-by-stop-of-ResourceSharer4Xi to: " + str(self._listener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    for key, (send, close) in self._cache.items():
                        close()
                    
                    message2print = "{(Step 2/3n) performed" + "; close-of-pairs-performed-by-stop-of-ResourceSharer4Xi: " + str(self._cache.items)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #     
                        
                    self._cache.clear()
                    message2print = "{(Step 2/3o) performed" + "; cache-clear-performed-by-stop-of-ResourceSharer4Xi: " + str(self._cache.clear)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #   
                
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 4/4) returning" + "; stop-of-ResourceSharer4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
            
            return
        
    #</methodize4xi>                
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_afterfork-of-ResourceSharer4Xi" usage = "_afterforks with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _afterfork(self):
        '''
        _afterforks with (self)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3) - NOT PERFORMED            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "_afterfork"
        
        message2print = "{(Step 1/3a) started" + "; _afterfork-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-afterfork-of-Listener4Xi: " + str(ResourceSharer._afterfork) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            ResourceSharer._afterfork(self)
            
            message2print = "{(StepOriginal) performed" + "; Original-ResourceSharer-performed-by-afterfork-of-Listener4Xi: " + str(ResourceSharer._afterfork) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #                
        else:
                            
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)            
            # -----------------------------------------------------------------------------------------            
            
            for key, (send, close) in self._cache.items():
                
                message2print = "{(Step 2/3a) using" + "; key-using-by-_afterfork-of-ResourceSharer4Xi: " + str(key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
                
                message2print = "{(Step 2/3b) using" + "; send-using-by-_afterfork-of-ResourceSharer4Xi: " + str(send) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
                
                message2print = "{(Step 2/3c) using" + "; close-using-by-_afterfork-of-ResourceSharer4Xi: " + str(close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
            
                
                close()
             
            message2print = "{(Step 2/3d) performing" + "; clear-performing-by-_afterfork-of-ResourceSharer4Xi: " + str(self._cache.clear) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #    
                   
            self._cache.clear()
            
            message2print = "{(Step 2/3e) performed" + "; clear-performed-by-_afterfork-of-ResourceSharer4Xi: " + str(self._cache.clear) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  
            
            # If self._lock was locked at the time of the fork, it may be broken
            # -- see issue 6721.  Replace it without letting it be gc'ed.
            
            message2print = "{(Step 2/3f) appending" + "; lock-appending-by-_afterfork-of-ResourceSharer4Xi: " + str(self._lock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            self._old_locks.append(self._lock)
            
            message2print = "{(Step 2/3g) appended" + "; lock-appended-by-_afterfork-of-ResourceSharer4Xi: " + str(self._lock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            self._lock = threading.Lock()
            
            message2print = "{(Step 2/3h) set" + "; lock-set-by-_afterfork-of-ResourceSharer4Xi to: " + str(self._lock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            if self._listener is not None:
                self._listener.close()
                
                message2print = "{(Step 2/3i) performed" + "; close-performed-by-_afterfork-of-ResourceSharer4Xi: " + str(self._listener.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
            
            self._listener = None
            message2print = "{(Step 2/3j) set" + "; lock-set-by-_afterfork-of-ResourceSharer4Xi to: " + str(self._lock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            
            self._address = None
            message2print = "{(Step 2/3k) set" + "; address-set-by-_afterfork-of-ResourceSharer4Xi to: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            self._thread = None
            message2print = "{(Step 2/3l) set" + "; thread-set-by-_afterfork-of-ResourceSharer4Xi to: " + str(self._thread) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 4/4) returning" + "; _afterfork-of-ResourceSharer4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
            
            return
        
    #</methodize4xi>                
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_start-of-ResourceSharer4Xi" usage = "_starts with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _start(self):
        '''
        _starts with (self)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "_start"
        
        message2print = "{(Step 1/3a) started" + "; _start-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 08:50:41 03/14/13 Pacific Daylight Time
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-start-of-Listener4Xi: " + str(ResourceSharer._start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            
            ResourceSharer._start(self)
            
            message2print = "{(StepOriginal) performed" + "; Original-ResourceSharer-performed-by-start-of-Listener4Xi: " + str(ResourceSharer._start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #               
        else:
                           
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) -          
            # -----------------------------------------------------------------------------------------            
            assert self._listener is None
            
            authkey2xi = get_authkey_current_process4xi()        
            message2print = "{(Step 2/3a) got" + "; authkey2xi-got-by-start-of-Listener4Xi: " + str(authkey2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # b'\xc6\x85\x04\xd1J\xae\x0f"\xf7w\xf3A\xdf\x9a\xce\xea\xf4B\xb4\xf1EsK\x96\xb2\xa7I0\x19U\xc1O'
            
            self._listener = Listener4Xi(authkey=authkey2xi)
            
            message2print = "{(Step 2/3b) set" + "; listener-set-by-start-of-Listener4Xi to: " + str(self._listener) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <listener4xi.Listener4Xi object at 0x0000000003630B00>
            
            self._address = self._listener.address
            message2print = "{(Step 2/3c) set" + "; address-set-by-start-of-Listener4Xi to: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # \\.\pipe\pyc-3824-0-fi0jby
            
            thread2xi = threading.Thread(target=self._serve)
            message2print = "{(Step 2/3d) set" + "; thread2xi-set-by-start-of-Listener4Xi to: " + str(thread2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <Thread(Thread-1, initial)>
            
            thread2xi.daemon = True
            message2print = "{(Step 2/3e) set" + "; daemon-set-by-start-of-Listener4Xi to: " + str(thread2xi.daemon) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # True
            
           
            thread2xi.start()
            
                        
            message2print = "{(Step 2/3f) started" + "; thread2xi-started-by-start-of-Listener4Xi: " + str(thread2xi.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <bound method Thread.start of <Thread(Thread-1, started daemon 5088)>>
            
            
            self._thread = thread2xi
            message2print = "{(Step 2/3g) set" + "; thread-set-by-start-of-Listener4Xi to: " + str(self._thread) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <Thread(Thread-1, started daemon 5088)>
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 3/3) returning" + "; _start-of-ResourceSharer4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
            # 09:54:26 03/14/13 Pacific Daylight Time
            
            return
        
    #</methodize4xi>                
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_serve-of-ResourceSharer4Xi" usage = "_serves with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _serve(self):
        '''
        _serves with (self)
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)            
        # -----------------------------------------------------------------------------------------
        class2print = "ResourceSharer4Xi"
        method2print = "_serve"
        
        message2print = "{(Step 1/3a) started" + "; _serve-of-ResourceSharer4Xi-started: " + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)   
        # 15:29:48 03/14/13 FLE Standard Time 
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-ResourceSharer-performing-by-serve-of-Listener4Xi: " + str(ResourceSharer._serve) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            
            ResourceSharer._serve(self)
            
            message2print = "{(StepOriginal) performed" + "; Original-ResourceSharer-performed-by-serve-of-Listener4Xi: " + str(ResourceSharer._serve) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #                
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) -            
            # -----------------------------------------------------------------------------------------            
            if hasattr(signal, 'pthread_sigmask'):
                signal.pthread_sigmask(signal.SIG_BLOCK, range(1, signal.NSIG))
               
                message2print = "{(Step 2/3a) using" + "; pthread_sigmask-using-by-_serve-of-ResourceSharer4Xi: " + str('pthread_sigmask') + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #
            else:
                message2print = "{(Step 2/3b) using" + "; mask2nonsignal-using-by-_serve-of-ResourceSharer4Xi: " + str('mask2nonsignal') + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 09:54:26 03/14/13 Pacific Daylight Time               
                
            while 1:
                try:
                    conn2server = self._listener.accept()
                    # accept-of-Listener4Xi-started
                    # accept-of-PipeListener4Xi-started
                    
                    
                    # JATKA - why does it not get connection server ?
                    
                    message2print = "{(Step 2/3b) got" + "; conn2server-got-by-_serve-of-ResourceSharer4Xi: " + str(conn2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # None  
                    
                    msg2server = conn2server.recv()
                    message2print = "{(Step 2/c) got" + "; msg2server-got-by-_serve-of-ResourceSharer4Xi: " + str(msg2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    if msg2server is None:
                        break
                        
                    key2server, destination_pid2server = msg2server
                    message2print = "{(Step 2/d) got" + "; key2server-got-by-_serve-of-ResourceSharer4Xi: " + str(key2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    message2print = "{(Step 2/e) got" + "; destination_pid2server-got-by-_serve-of-ResourceSharer4Xi: " + str(destination_pid2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    send2server, send2server = self._cache.pop(key2server)
                    
                    message2print = "{(Step 2/f) set" + "; send2server-set-by-_serve-of-ResourceSharer4Xi to : " + str(send2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    send2server(conn2server, destination_pid2server)
                    message2print = "{(Step 2/g) performed" + "; send2server-performed-by-_serve-of-ResourceSharer4Xi: " + str(send2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    send2server()
                    message2print = "{(Step 2/h) performed" + "; send2server-performed-by-_serve-of-ResourceSharer4Xi: " + str(send2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    conn2server.send2server()
                    
                    message2print = "{(Step 2/i) performed" + "; send2server-performed-by-_serve-of-ResourceSharer4Xi: " + str(conn2server.send2server) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # 
                    
                    
                except:
                    if not is_exiting4xi():
                        import traceback
                        sub_warning(
                        'thread for sharing handles raised exception :\n' +
                        '-'*79 + '\n' + traceback.format_exc() + '-'*79)                   
                        message2print = "{(Step 2/z) warning" + "; exception-warning-by-_serve-of-ResourceSharer4Xi: " + str('** thread for sharing handles raised exception**') + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        #     
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 4/4) returning" + "; _serve-of-ResourceSharer4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
            
            return
        
    #</methodize4xi>
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ResourceSharer4Xi" usage = "senses ResourceSharer4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ResourceSharer4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = 'ResourceSharer4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-ResourceSharer4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; ResourceSharer4Xi-sensing-by-sense-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-ResourceSharer4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; ResourceSharer4Xi-sensed-by-sense-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-ResourceSharer4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
      

        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ResourceSharer4Xi" usage = "reasons ResourceSharer4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ResourceSharer4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = 'ResourceSharer4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-ResourceSharer4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; ResourceSharer4Xi-reasoning-by-reason-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-ResourceSharer4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{(Step 4/5) reasoning" + "; ResourceSharer4Xi-reasoning-by-reason-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-ResourceSharer4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
         

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ResourceSharer4Xi" usage = "aces ResourceSharer4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ResourceSharer4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = 'ResourceSharer4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-ResourceSharer4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; ResourceSharer4Xi-acing-by-ace-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-ResourceSharer4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; ResourceSharer4Xi-acing-by-ace-of-ResourceSharer4Xi: " + str(ResourceSharer4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-ResourceSharer4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of resourcesharer4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

